@@ -38,14 +38,15 @@ import FastPackedString ( PackedString, readFilePS, gzReadFilePS, writeFilePS,
 import Lock ( withTemp, withOpenTemp, readDocBinFile,
               canonFilename, writeBinFile, writeDocBinFile
             )
-import Autoconf ( have_libcurl, have_sendmail, have_mapi, sendmail_path )
+import CommandLine ( parseCmd, addUrlencoded )
+import Autoconf ( have_libcurl, have_sendmail, have_mapi, sendmail_path, darcs_version )
 import Curl ( copyUrl )
 import Curl ( Cachable(..) )
 import Exec ( exec )
 import DarcsURL ( is_file, is_url, is_ssh )
 import DarcsUtils ( catchall )
 import Printer ( Doc, hPutDoc, hPutDocLn, ($$), (<+>), renderPS,
-                 text, empty, packedString, vcat )
+                 text, empty, packedString, vcat, renderString )
 #include "impossible.h"
 
 
@@ -189,17 +190,27 @@ copySSHs u ns d = do sftp_command <- getEnv "DARCS_SFTP" `catch`
                                      \_ -> return "sftp"
                      let sftp = head $ words sftp_command
                          sftp_args = tail $ words sftp_command
-                         input = unlines $ map ("get "++) ns
+                         path = drop 1 $ dropWhile (/= ':') u
+                         host = (takeWhile (/= ':') u)++":"
+                         cd = "cd "++path++"\n"
+                         input = cd++(unlines $ map ("get "++) ns)
                      withCurrentDirectory d $ withOpenTemp $ \(th,tn) ->
                          do hPutStr th input
                             hClose th
-                            r <- exec sftp (sftp_args++["-b",tn,u])
+                            r <- exec sftp (sftp_args++["-b",tn,host])
                                  "/dev/null" "/dev/null"
+                            let files = if length ns > 5
+                                          then (take 5 ns) ++ ["and "
+                                               ++ (show (length ns - 5)) ++ " more"]
+                                          else ns
+                                hint = if take 1 path == "~"
+                                         then ["sftp doesn't expand ~, use path/ instead of ~/path/"]
+                                         else []
                             when (r /= ExitSuccess) $
                                  fail $ unlines $
                                           ["(sftp) failed to fetch files.",
-                                           "source directory: " ++ d,
-                                           "source files:"] ++ ns
+                                           "source directory: " ++ path,
+                                           "source files:"] ++ files ++ hint
 
 
 copyRemoteCmd :: String -> FilePath -> IO ()
@@ -221,7 +232,7 @@ doWithPatches opts f patches =
 
               doWithCount :: [String] -> Int -> IO ()
               doWithCount (p:ps) index = do
-                  putStr $ if verbose then "Copying patch "++p++"\n"
+                  putStr $ if verbose then "Copying patch "++(show index)++" of "++total++": "++p++"\n"
                            else "\rCopying patch "++(show index)++
                                 " of "++total++"..."
                   hFlush stdout
@@ -271,11 +282,11 @@ pipeDoc_SSH_IgnoreError args input =
        execPipeIgnoreError ssh (ssh_args++port++args) input
 
 
-sendEmail :: String -> String -> String -> String -> String -> IO ()
-sendEmail _ "" _ "" _ = return ()
-sendEmail f "" s cc body = sendEmail f cc s "" body
-sendEmail f t s cc body =
-  if have_sendmail then do
+sendEmail :: String -> String -> String -> String -> String -> String -> IO ()
+sendEmail _ "" _ "" _ _ = return ()
+sendEmail f "" s cc scmd body = sendEmail f cc s "" scmd body
+sendEmail f t s cc scmd body =
+  if have_sendmail || scmd /= "" then do
     withOpenTemp $ \(h,fn) -> do
      hPutStr h $
         "To: "      ++ t ++ "\n" ++
@@ -283,9 +294,11 @@ sendEmail f t s cc body =
         "Subject: " ++ s ++ "\n" ++
         formated_cc ++
         "X-Mail-Originator: Darcs Version Control System\n" ++
+        "X-Darcs-Version: " ++ darcs_version ++ "\n" ++
         body
      hClose h
-     r <- exec sendmail_path ["-i", "-t"] fn "/dev/null"
+     let ftable = [('t',t),('c',cc),('t',fn),('f',f),('s',s),('b',body)]
+     r <- execSendmail ftable scmd fn
      when (r /= ExitSuccess) $ fail ("failed to send mail to: " ++ t)
   else if have_mapi then do
      r <- withCString t $ \tp ->
@@ -305,11 +318,13 @@ sendEmail f t s cc body =
                       then ""
                       else "Cc: "++cc++"\n"
 
-sendEmailDoc :: String -> String -> String -> String -> Doc -> IO ()
-sendEmailDoc _ "" _ "" _ = return ()
-sendEmailDoc f "" s cc body = sendEmailDoc f cc s "" body
-sendEmailDoc f t s cc body =
-  if have_sendmail then do
+sendEmailDoc :: String -> String -> String -> String -> String
+             -> Doc -> Doc -> Doc -> IO ()
+sendEmailDoc _ "" _ "" _ _ _ _ = return ()
+sendEmailDoc f "" s cc scmd content bundle body =
+  sendEmailDoc f cc s "" scmd content bundle body
+sendEmailDoc f t s cc scmd content bundle body =
+  if have_sendmail || scmd /= "" then do
     withOpenTemp $ \(h,fn) -> do
      hPutDocLn h $
            text "To:"      <+> text t
@@ -317,10 +332,16 @@ sendEmailDoc f t s cc body =
         $$ text "Subject:" <+> text s
         $$ formated_cc
         $$ text "X-Mail-Originator: Darcs Version Control System"
+        $$ text ("X-Darcs-Version: " ++ darcs_version)
         $$ body
      hClose h
-     r <- exec sendmail_path ["-i", "-t"] fn "/dev/null"
-     when (r /= ExitSuccess) $ fail ("failed to send mail to: " ++ t)
+     withOpenTemp $ \(hat,at) -> do
+       hPutDocLn hat $ bundle
+       hClose hat
+       let ftable = [('t',t),('c',cc),('f',f),('s',s),
+                     ('b',renderString content), ('a', at)]
+       r <- execSendmail ftable scmd fn
+       when (r /= ExitSuccess) $ fail ("failed to send mail to: " ++ t)
   else if have_mapi then do
      r <- withCString t $ \tp ->
            withCString f $ \fp ->
@@ -339,10 +360,10 @@ sendEmailDoc f t s cc body =
                       then empty
                       else text "Cc:" <+> text cc
 
-resendEmail :: String -> PackedString -> IO ()
-resendEmail "" _ = return ()
-resendEmail t body =
-  case (have_sendmail, have_mapi) of
+resendEmail :: String -> String -> PackedString -> IO ()
+resendEmail "" _ _ = return ()
+resendEmail t scmd body =
+  case (have_sendmail || scmd /= "", have_mapi) of
    (True, _) -> do
     withOpenTemp $ \(h,fn) -> do
      hPutStrLn h $ "To: "++ t
@@ -350,10 +371,11 @@ resendEmail t body =
      hPutStrLn h $ find_subject (linesPS body)
      hPutDocLn h $ fixit $ linesPS body
      hClose h
-     r <- exec sendmail_path ["-i", "-t"] fn "/dev/null"
+     let ftable = [('t',t)]
+     r <-  execSendmail ftable scmd fn
      when (r /= ExitSuccess) $ fail ("failed to send mail to: " ++ t)
    (_, True) -> fail "Don't know how to resend email with MAPI"
-   _ -> fail $ "no mail facility (sendmail or mapi) located at configure time!"
+   _ -> fail $ "no mail facility (sendmail or mapi) located at configure time (use the sendmail-command option)!"
   where br = packString "\r"
         darcsurl = packString "DarcsURL:"
         content = packString "Content-"
@@ -373,6 +395,17 @@ resendEmail t body =
                             | otherwise = find_subject ls
         find_subject [] = "Subject: (no subject)"
 
+execSendmail :: [(Char,String)] -> String -> String -> IO ExitCode
+execSendmail ftable scmd fn =
+  if scmd == "" then
+     exec sendmail_path ["-i", "-t"] fn "/dev/null"
+  else case parseCmd (addUrlencoded ftable) scmd of
+         Right (arg0:opts, wantstdin) ->
+           do let stdin = if wantstdin then fn else "/dev/null"
+              exec arg0 opts stdin "/dev/nulll"
+         Left e -> fail $ ("failed to send mail, invalid sendmail-command: "++(show e))
+         _ -> fail $ ("failed to send mail, invalid sendmail-command")
+
 #ifdef HAVE_MAPI
 foreign import ccall "win32/send_email.h send_email" c_send_email
 #else
