@@ -20,18 +20,25 @@ import Repository ( read_repo )
 import PatchInfo ( pi_author )
 
 import Data.List ( sort, group )
+import System ( getProgName, getArgs )
+import Monad ( liftM )
 
-intro :: String
-intro =
-  "This is a list of ALL contributors to darcs, sorted according to the\n"++
-  "number of patches they have contributed.  This list is automatically\n"++
-  "created from the darcs repository, so if there are problems with it,\n"++
-  "changes should be made to list_authors.hs.\n"
+intro :: IO String
+intro = do
+  prog_name <- getProgName
+  return $
+    "This is a list of ALL contributors to darcs, sorted according to the\n"++
+    "number of patches they have contributed.  This list is automatically\n"++
+    "created from the darcs repository, so if there are problems with it,\n"++
+    "changes should be made to list_authors.hs.\n"++
+    "Say \"" ++ prog_name ++ " stats\" for more detailed information.\n"++
+    "\n"
 
 main :: IO ()
 main = do darcs_history <- read_repo "."
-          putStrLn intro
-          mapM_ putStrLn $ sort_authors $ map (pi_author.fst)
+          intro >>= putStr
+          use_statistics <- elem "stats" `liftM` getArgs
+          mapM_ putStrLn $ sort_authors use_statistics $ map (pi_author.fst)
                          $ concat darcs_history
 
 droundy :: String
@@ -54,16 +61,34 @@ canonize_author "Tomasz Zielonka <t.zielonka@students.mimuw.edu.pl>"
     = "Tomasz Zielonka <tomasz.zielonka@gmail.com>"
 canonize_author "Tomasz Zielonka <tomekz@gemius.pl>"
     = "Tomasz Zielonka <tomasz.zielonka@gmail.com>"
+canonize_author "aj@azure.humbug.org.au"
+    = "Anthony Towns <aj@azure.humbug.org.au>"
+canonize_author "peter@zarquon.se" = "Peter Strand <peter@zarquon.se>"
+canonize_author "schnetter@aei.mpg.de"
+    = "Erik Schnetter <schnetter@aei.mpg.de>"
+canonize_author "fw@deneb.enyo.de" = "Florian Weimer <fw@deneb.enyo.de>"
+canonize_author "wnoise@ofb.net" = "Aaron Denney <wnoise@ofb.net>"
+canonize_author "kannan@cakoose.com" = "Kannan Goundan <kannan@cakoose.com>"
+canonize_author "zander@kde.org" = "Thomas Zander <zander@kde.org>"
+canonize_author "rho@swiftdsl.com.au" = "Nigel Rowe <rho@swiftdsl.com.au>"
+canonize_author "ptp@lysator.liu.se" = "Tommy Pettersson <ptp@lysator.liu.se>"
 canonize_author s
     | s `contains` "roundy" = droundy
     | s `contains` "igloo" = "Ian Lynagh <igloo@earth.li>"
+    | s `contains` "Thomas Zander" = "Thomas Zander <zander@kde.org>"
+    | s `contains` "marnix.klooster@" = "Marnix Klooster <marnix.klooster@gmail.com>"
+    | s `contains` "mklooster@" = "Marnix Klooster <marnix.klooster@gmail.com>"
     | s `contains` "mirian@" = "Mirian Crzig Lennox <mirian@cosmic.com>"
     | s `contains` "zbrown" = "Zack Brown <zbrown@tumblerings.org>"
     | otherwise = s
 
-sort_authors :: [String] -> [String]
-sort_authors as = reverse $ map snd $ sort $ map (\s -> (length s,head s)) $
-                  group $ sort $ map canonize_author as
+sort_authors :: Bool -> [String] -> [String]
+sort_authors use_stats as = reverse $ map shownames $ sort $
+                            map (\s -> (length s,head s)) $
+                            group $ sort $ map canonize_author as
+        where shownames (n, a) = if use_stats
+                                 then show n ++ "\t" ++ a
+                                 else a
 
 contains :: String -> String -> Bool
 a `contains` b | length a < length b = False
