@@ -43,6 +43,18 @@ like(`$DARCS changes date.t`,qr/foo/,'after 50 records: changes file.txt: trivia
 like(`$DARCS changes --last=1 date.t`,qr/foo/,'after 50 records: changes --last=1 file.txt');
 like(`$DARCS changes --last=1 --summary date.t`,qr/foo/,'after 50 records: changes --last=1 --summary file.txt');
 
+###
+
+`date >>second_file.t`;
+`darcs add second_file.t`;
+like(`darcs record -A x -a -m adding_second_file second_file.t 2>&1`, qr/finished recording/i, 'recorded second file');
+
+TODO: {
+    local $TODO = 'still a future feature.';
+    my $test_name = '--last should mean "last N affecting this file"';
+    like(`darcs changes --last=1 date.t`,qr/foo/,$test_name);
+}
+
 
 chdir '../';
 `rm -rf temp1`;
