@@ -12,18 +12,18 @@ $DARCS add foo
 $DARCS record -a -m a -A test
 echo b > foo
 echo y | $DARCS revert -a
-echo -e 'y\ny' | $DARCS unrecord
+echo y/d/y | tr / \\012 | $DARCS unrecord
 # since the unrevert is impossible, we should fail if it succeeds...
 ! echo y | $DARCS unrevert
 
 # now let's try a possible unrevert, just for fun...
 echo b >> foo
 $DARCS record -a -m b -A test
-echo -e 'f\nb' > foo
+echo f/b | tr / \\012 > foo
 $DARCS record -a -m 'aaa becomes f' -A test
 date >> foo
 echo y | $DARCS revert -a
-echo y | $DARCS unpull
+echo y/d/y | tr / \\012 | $DARCS unpull
 # Now add the date back on at the end:
 echo y | $DARCS unrevert
 echo 'M ./foo +1' > correct_summary
