@@ -3,19 +3,19 @@
 # Some tests for the behavior when there is a conflict in _darcs/patches/pending
 
 use Test::More qw/no_plan/;
+use lib ('lib/perl');
+use Test::Darcs;
+use Shell::Command;
 use strict;
 use vars qw/$DARCS/;
 
 die 'darcs not found' unless $ENV{DARCS} || (-x "$ENV{PWD}/../darcs");
 $DARCS = $ENV{DARCS} || "$ENV{PWD}/../darcs";
 
-
-
-`rm -rf temp1`;
-`mkdir temp1`;
+rm_rf  'temp1';
+mkpath 'temp1';
 chdir 'temp1';
-`$DARCS init`;
-#`echo "Tester" > _darcs/prefs/author`;
+darcs 'init';
 open(PENDING,'>_darcs/patches/pending') || die "couldn't open pending: $!";
 print PENDING '{
     addfile ./date.t
@@ -25,7 +25,7 @@ print PENDING '{
 close(PENDING); 
 
 # now watch the fireworks as all sorts of things fail
-like(`$DARCS whatsnew`, qr/pending has conflicts/, "darcs whatsnew reports 'pending has conflicts'");
+like( darcs ('whatsnew'), qr/pending has conflicts/, "darcs whatsnew reports 'pending has conflicts'");
 
 ####
 
@@ -34,13 +34,13 @@ like($revert_output, qr/pending has conflicts/, 'darcs revert reports "pending h
 
 ###
 
-my $record_output = `$DARCS record -A x -a -m foo`;
+my $record_output =  darcs(qw/record -A x -a -m foo/);
 like($record_output, qr/pending has conflicts/, 'darcs record reports "pending has conflicts"');
 
-like(`$DARCS repair`, qr/The repo is already consistent, no changes made/i, 'darcs repair finds no problem');
+like( darcs('repair'), qr/The repo is already consistent, no changes made/i, 'darcs repair finds no problem');
 
 chdir '../';
-`rm -rf temp1`;
+rm_rf 'temp1';
 ok((!-d 'temp1'), 'temp1 directory was deleted');
 
 
