@@ -16,7 +16,7 @@ $DARCS mv temp.c d/
 $DARCS record --all -A test --patch-name=mvetc
 cmp d/temp.c _darcs/current/d/temp.c
 
-echo y | $DARCS unrecord
+echo y/d/y | tr / \\012 | $DARCS unrecord
 $DARCS whatsnew
 cmp d/temp.c _darcs/current/temp.c
 
