@@ -73,7 +73,8 @@ int gzreader(char *f, int *thid) {
   hdl = CreateThread(NULL, 0, the_reader, th, 0, &temp);
   if (!hdl) {
     gzclose(th->gzf);
-    close(th->fd);
+    close(fd[0]);
+    close(fd[1]);
     free(th);
     return -2;
   }
@@ -81,7 +82,8 @@ int gzreader(char *f, int *thid) {
 #else
   if (pthread_create(&the_thid, NULL, the_reader, th)) {
     gzclose(th->gzf);
-    close(th->fd);
+    close(fd[0]);
+    close(fd[1]);
     free(th);
     return -2;
   }
@@ -98,7 +100,7 @@ static void *the_writer(void *void_th) {
   char buf[bufsize];
   thread_data *th = (thread_data *)void_th;
   int bytesread;
-  while (bytesread = read(th->fd,buf,bufsize)) {
+  while ((bytesread = read(th->fd,buf,bufsize))) {
     int byteswritten = 0;
     while (byteswritten < bytesread) {
       int morebytes = gzwrite(th->gzf, buf+byteswritten,
@@ -138,7 +140,8 @@ int gzwriter(char *f, int *thid) {
   hdl = CreateThread(NULL, 0, the_writer, th, 0, &temp);
   if (!hdl) {
     gzclose(th->gzf);
-    close(th->fd);
+    close(fd[0]);
+    close(fd[1]);
     free(th);
     return -2;
   }
@@ -146,7 +149,8 @@ int gzwriter(char *f, int *thid) {
 #else
   if (pthread_create(&the_thid, NULL, the_writer, th)) {
     gzclose(th->gzf);
-    close(th->fd);
+    close(fd[0]);
+    close(fd[1]);
     return -2;
   }
   *thid = the_thid;
@@ -161,7 +165,7 @@ int wait_on_gz(int thid) {
   if (joinval == 0)
     GetExitCodeThread((HANDLE) thid, (LPDWORD)&retval);
 #else
-  void* retval;
+  void *retval;
   int joinval = pthread_join(thid, &retval);
 #endif
   if (joinval) return joinval;
